# R Script for downloading the Lyft Bay Wheels data
#
# Version 2

start_year <- 19
current_year <- 20
first_month <- 5
last_month <- 8

library(pacman)
p_load(filesstrings, fs, glue, readbulk, DataExplorer, tictoc, skimr, lubridate, forcats, tidyverse, feather)

# Note: Even using a space after if and for statements.

if (dir_exists("data")) dir_delete("data")
if (!dir_exists("data")) dir_create("data")

# Test the loop over the years and months.

for (j in start_year:current_year) {
  message('Year: ',j,'\n')
# loop for start_year and first_month  
if (j == start_year){
    for (i in first_month:12) {
    message('Month: ',i,'\n')
    }  
  }  
# loop for middle years and months  
if (j > start_year & j < current_year){
  message('Year: ',j,'\n')
    for (i in 1:12) {
      message('Month: ',i,'\n')
    }  
  }  
# loop for current year and last_month  
if (j == current_year){
  message('Year: ',j,'\n')
    for (i in 1:last_month) {
      message('Month: ',i,'\n')
    }  
  }   
}

# Download the files using the loop over the years and months.

tic()
for (j in start_year:current_year) {
  # loop for start_year and first_month  
  if (j == start_year){
    message('Year: ',j,'\n')
    for (i in first_month:12) {
      message('Month: ',i,'\n')
      URL <- glue('https://s3.amazonaws.com/baywheels-data/20{j}{str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip')
      download.file(URL, destfile = glue('./data/20{j}{str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip'))
      unzip(glue('./data/20{j}{str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip'),exdir="./data")
      fn <- glue('./data/20{j}{str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip')
      if (file_exists(fn)) file_delete(fn)
      
    }  
  }  
  # loop for middle years and months  
  if (j > start_year & j < current_year){
    message('Year: ',j,'\n')
    for (i in 1:12) {
      message('Month: ',i,'\n')
      URL <- glue('https://s3.amazonaws.com/baywheels-data/20{j}{str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip')
      download.file(URL, destfile = glue('./data/20{j}{str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip'))
      unzip(glue('./data/20{j}{str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip'),exdir="./data")
      fn <- glue('./data/20{j}{str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip')
      if (file_exists(fn)) file_delete(fn)
    }  
  }  
  # loop for current year and last_month  
  if (j == current_year){
    message('Year: ',j,'\n')
    for (i in 1:last_month) {
      message('Month: ',i,'\n')
      URL <- glue('https://s3.amazonaws.com/baywheels-data/20{j}{str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip')
      download.file(URL, destfile = glue('./data/20{j}{str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip'))
      unzip(glue('./data/20{j}{str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip'),exdir="./data")
      fn <- glue('./data/20{j}{str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip')
      if (file_exists(fn)) file_delete(fn)
    }  
  }   
}
toc()

dir_delete("data/__MACOSX")

# Fix columns names in the .csv files starting in 202004.

if (dir_exists("data2")) remove_dir("data2")
if (!dir_exists("data2")) create_dir("data2")

start_year <- 20
current_year <- 20
first_month <- 4
last_month <- 8

for (j in start_year:current_year) {
    for (i in first_month:last_month) {
       file_move(glue('./data/20{j}{str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv'),
                 './data2/')
    }  
}   

# Slowly read each .csv file into one tibble.

tic()
lyftbaywheels01 <- readbulk::read_bulk(directory = "data", 
                                       extension = ".csv",
                                       fun=read.csv2,
                                       stringsAsFactors=FALSE) %>% as_tibble()
toc()

tic()
lyftbaywheels02 <- readbulk::read_bulk(directory = "data2", 
                                       extension = ".csv",
                                       fun=read.csv2,
                                       stringsAsFactors=FALSE) %>% as_tibble()
toc()

names(lyftbaywheels01)
names(lyftbaywheels02)

DataExplorer::introduce(lyftbaywheels01)
DataExplorer::introduce(lyftbaywheels02)

# Test out the map function.  Is this faster?  First into a tibble.  Second into a data.table.
# Read this StackOverflow page about this [import multipl csv files](https://stackoverflow.com/questions/11433432/how-to-import-multiple-csv-files-at-once)

tic()
lyftbaywheels01c <-
  list.files(path = "./data/",
             pattern = "*.csv", 
             full.names = T) %>% 
  map_df(~read_csv(., col_types = cols(.default = "c"))) 

lyftbaywheels01c <- lyftbaywheels01c %>% mutate(
                      start_station_id = as.integer(start_station_id),
                      end_station_id = as.integer(end_station_id)
                      )
toc()

tic()
lyftbaywheels02c <-
  list.files(path = "./data2/",
             pattern = "*.csv", 
             full.names = T) %>% 
  map_df(~read_csv(., col_types = cols(.default = "c"))) 

lyftbaywheels02c <- lyftbaywheels02c %>% mutate(
                      start_station_id = as.integer(start_station_id),
                      end_station_id = as.integer(end_station_id)
                     )
toc()

p_load(purrr, data.table, dtplyr)

# The map() function produces a list().  There is a problem again with the _id variables.
# The map_df() function outputs one tibble.

tic()
lyftbaywheels01d <- 
  list.files(path = "./data/",
             pattern = "*.csv", 
             full.names = T) %>% 
  map(~fread(., stringsAsFactors=FALSE))

lyftbaywheels01d[[1]] <- lyftbaywheels01d[[1]] %>% lazy_dt() %>% 
                          mutate(
                            start_station_id = as.integer(start_station_id),
                            end_station_id = as.integer(end_station_id)
                          ) %>%
                          as.data.table()

lyftbaywheels01d <- lyftbaywheels01d %>% map_df(~rbind(.))
toc()

tic()
lyftbaywheels02d <- 
  list.files(path = "./data2/",
             pattern = "*.csv", 
             full.names = T) %>% 
  map_df(~fread(., stringsAsFactors=FALSE))

lyftbaywheels02d <- lyftbaywheels02d %>% lazy_dt() %>% 
                      mutate(
                        start_station_id = as.integer(start_station_id),
                        end_station_id = as.integer(end_station_id)
                      ) %>%
                      as.data.table()
toc()

# Fix the date time variables

lyftbaywheels01 <- lyftbaywheels01 %>% mutate(
 start_time = as_datetime(start_time), 
 end_time = as_datetime(end_time)
)

# Fix the data from 202004 to have the same variables names as before.
# And fix the date time variables.

lyftbaywheels02a <- lyftbaywheels02 %>% rename(
  "start_time" = "started_at",
  "end_time" = "ended_at",
  "start_station_latitude" = "start_lat",
  "start_station_longitude" = "start_lng",
  "end_station_latitude" = "end_lat",
  "end_station_longitude" = "end_lng"
) %>% mutate(
  start_time = as_datetime(start_time), 
  end_time = as_datetime(end_time)
)

names(lyftbaywheels02a)

# Fix the values of user_type, calculate the duration variable, fix station_id.

lyftbaywheels02b <- lyftbaywheels02a %>% 
  rename("user_type" = "member_casual")  %>% 
  mutate(user_type = as_factor(user_type)) %>% 
  mutate(user_type = fct_recode( user_type,
                            Customer = "casual", Subscriber = "member") ) %>% 
  mutate(user_type = as.character(user_type)) %>%
  mutate(duration_sec = 
           as.numeric(difftime(end_time, start_time, units = "secs"))) %>%
  mutate(
    start_station_id = as.character(start_station_id),
    end_station_id = as.character(end_station_id)
  )

# row_bind the data together.

lyftbaywheels_final <- bind_rows(lyftbaywheels01, lyftbaywheels02b)

names(lyftbaywheels_final)

DataExplorer::introduce(lyftbaywheels_final)

# Add year, month, day, and day of the week variables.

lyftbaywheels_final <- lyftbaywheels_final %>% mutate(year=year(start_time), 
                                   month=month(start_time), 
                                   day=day(start_time) )

lyftbaywheels_final <- lyftbaywheels_final %>% mutate(week_day = wday(start_time) )

levels <- c("M","T","W","TH","F","SAT","SUN")

lyftbaywheels_final <- lyftbaywheels_final %>% mutate( week_day = factor(week_day, levels = levels) )

names(lyftbaywheels_final)

DataExplorer::introduce(lyftbaywheels_final)


# Test out some different ways to write a dataframe to the harddrive.

if (dir_exists("data_final")) dir_delete("data_final")
if (!dir_exists("data_final")) dir_create("data_final")

# write .Rds

tic()
saveRDS(lyftbaywheels_final, "data_final/lyftbaywheels_final.Rds")
toc()

# write .feather from feather R package

tic()
write_feather(lyftbaywheels_final, "data_final/lyftbaywheels_final.feather")
toc()

# remove all files

dir_delete("data")
dir_delete("data2")
