# R Script for downloading the Lyft Bay Wheels data
#
# Version 5

# In this code parallel downloading is implemented using the doFuture R package
#   and more output data files are created in the data_final directory.
#   An .Rds file is saved using the readr R package.
#   A  .csv file is saved using the readr R package
#   An .sqlite database is saved using the dbplyr R package.
#   A  .parquet file is saved using the arrow R package.
#   A  .feather file is saved using the arrow R package.
#   A data.frame set of files is saved using the data.frame R package.  This 
#     package save the data to files on your harddrive.  This is usefull if
#     you are working with a huge dataset that is beyond your computer's RAM.
#
# I have also tried to use all functions from the tidyverse.

start_year <- 19
current_year <- 20
first_month <- 5
last_month <- 8

library(pacman)
p_load(filesstrings, fs, glue, stringr, readbulk, DataExplorer, tictoc, skimr, lubridate, 
       forcats, tidyverse, arrow, dbplyr, disk.frame, beepr)

# Note: Even using a space after if and for statements.

if (dir_exists("data")) dir_delete("data")
if (!dir_exists("data")) dir_create("data")

# parallel loop

library(doFuture)
registerDoFuture()
plan(multisession)

# In parallel, test the loop over the years and months.

tic()
for (j in start_year:current_year)  {
  message('Year: ',j,'\n')
# loop for start_year and first_month  
if (j == start_year){
    foreach (i = first_month:12) %dopar% {
    message('Year: ',j,' Month: ',i,'\n')
    } 
  }  
# loop for middle years and months  
if (j > start_year & j < current_year){
  message('Year: ',j,'\n')
    foreach (i = 1:12) %dopar% {
      message('Year: ',j,' Month: ',i,'\n')
    }  
  }  
# loop for current year and last_month  
if (j == current_year){
  message('Year: ',j,'\n')
    foreach (i = 1:last_month) %dopar% {
      message('Year: ',j,' Month: ',i,'\n')
    }  
  }   
}

# In parallel, download the files using the loop over the years and months.

foreach (j = start_year:current_year) %do% {
  # loop for start_year and first_month  
  if (j == start_year){
    message('Year: ',j,'\n')
    foreach (i = first_month:12) %dopar% {
      message('Year: ',j,' Month: ',i,'\n')
      URL <- glue('https://s3.amazonaws.com/baywheels-data/20{j}{stringr::str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip')
      download.file(URL, destfile = glue('./data/20{j}{stringr::str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip'))
      unzip(glue('./data/20{j}{stringr::str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip'),exdir="./data")
      fn <- glue('./data/20{j}{stringr::str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip')
      if (file_exists(fn)) file_delete(fn)
    }  
  }  
  # loop for middle years and months  
  if (j > start_year & j < current_year){
    message('Year: ',j,'\n')
    foreach (i = 1:12) %dopar% {
      message('Year: ',j,' Month: ',i,'\n')
      URL <- glue('https://s3.amazonaws.com/baywheels-data/20{j}{stringr::str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip')
      download.file(URL, destfile = glue('./data/20{j}{stringr::str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip'))
      unzip(glue('./data/20{j}{stringr::str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip'),exdir="./data")
      fn <- glue('./data/20{j}{stringr::str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip')
      if (file_exists(fn)) file_delete(fn)
    }  
  }  
  # loop for current year and last_month  
  if (j == current_year){
    message('Year: ',j,'\n')
    foreach (i = 1:last_month) %dopar% {
      message('Year: ',j,' Month: ',i,'\n')
      URL <- glue('https://s3.amazonaws.com/baywheels-data/20{j}{stringr::str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip')
      download.file(URL, destfile = glue('./data/20{j}{stringr::str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip'))
      unzip(glue('./data/20{j}{stringr::str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip'),exdir="./data")
      fn <- glue('./data/20{j}{stringr::str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv.zip')
      if (file_exists(fn)) file_delete(fn)
    }  
  }   
}
toc()

dir_delete("data/__MACOSX")

# Fix columns names in the .csv files starting in 202004.

if (dir_exists("data2")) remove_dir("data2")
if (!dir_exists("data2")) create_dir("data2")

start_year <- 20
current_year <- 20
first_month <- 4
last_month <- 8

for (j in start_year:current_year) {
    for (i in first_month:last_month) {
       file_move(glue('./data/20{j}{stringr::str_pad(i, width=2, side="left", pad="0")}-baywheels-tripdata.csv'),
                 './data2/')
    }  
}   

# Test out the map function.  Is this faster?  First into a tibble.  Second into a data.table.
# Read this StackOverflow page about this [import multipl csv files](https://stackoverflow.com/questions/11433432/how-to-import-multiple-csv-files-at-once)

p_load(purrr, data.table, dtplyr)

# The map() function produces a list().  There is a problem again with the _id variables.
# The map_df() function outputs one tibble.
# Notice the use of the list.files() function.

tic()
lyftbaywheels01 <- 
  list.files(path = "./data/",
             pattern = "*.csv", 
             full.names = T) %>% 
  map(~fread(., stringsAsFactors=FALSE)) 

lyftbaywheels01[[1]] <- lyftbaywheels01[[1]] %>% lazy_dt() %>% 
                          mutate(
                            start_station_id = as.integer(start_station_id),
                            end_station_id = as.integer(end_station_id)
                          ) %>%
                          as.data.table()

lyftbaywheels01 <- lyftbaywheels01 %>% map_df(~rbind(.)) %>% as_tibble()
toc()

tic()
lyftbaywheels02 <- 
  list.files(path = "./data2/",
             pattern = "*.csv", 
             full.names = T) %>% 
  map_df(~fread(., stringsAsFactors=FALSE))

lyftbaywheels02 <- lyftbaywheels02 %>% lazy_dt() %>% 
                      mutate(
                        start_station_id = as.integer(start_station_id),
                        end_station_id = as.integer(end_station_id)
                      ) %>%
                      as.data.table() %>%
                      as_tibble()
toc()

names(lyftbaywheels01)
names(lyftbaywheels02)

DataExplorer::introduce(lyftbaywheels01)
DataExplorer::introduce(lyftbaywheels02)

# Fix the date time variables

lyftbaywheels01 <- lyftbaywheels01 %>% mutate(
 start_time = as_datetime(start_time), 
 end_time = as_datetime(end_time)
)

# Fix the data from 202004 to have the same variables names as before.
# And fix the date time variables.

lyftbaywheels02a <- lyftbaywheels02 %>% rename(
  "start_time" = "started_at",
  "end_time" = "ended_at",
  "start_station_latitude" = "start_lat",
  "start_station_longitude" = "start_lng",
  "end_station_latitude" = "end_lat",
  "end_station_longitude" = "end_lng"
) %>% mutate(
  start_time = as_datetime(start_time), 
  end_time = as_datetime(end_time)
)

names(lyftbaywheels02a)

# Fix the values of user_type, calculate the duration variable, fix station_id.

lyftbaywheels02b <- lyftbaywheels02a %>% 
  rename("user_type" = "member_casual")  %>% 
  mutate(user_type = as_factor(user_type)) %>% 
  mutate(user_type = fct_recode( user_type,
                            Customer = "casual", Subscriber = "member") ) %>% 
  mutate(user_type = as.character(user_type)) %>%
  mutate(duration_sec = 
           as.numeric(difftime(end_time, start_time, units = "secs"))) 

# row_bind the data together.

lyftbaywheels_final <- bind_rows(lyftbaywheels01, lyftbaywheels02b)

names(lyftbaywheels_final)

DataExplorer::introduce(lyftbaywheels_final)

# Add year, month, day, and day of the week variables.

lyftbaywheels_final <- lyftbaywheels_final %>% mutate(year=year(start_time), 
                                   month=month(start_time), 
                                   day=day(start_time) )

lyftbaywheels_final <- lyftbaywheels_final %>% mutate(week_day = wday(start_time) )

levels <- c("M","T","W","TH","F","SAT","SUN")

lyftbaywheels_final <- lyftbaywheels_final %>% mutate( week_day = factor(week_day, levels = levels) )

names(lyftbaywheels_final)

DataExplorer::introduce(lyftbaywheels_final)


# Test out some different ways to write a dataframe to the harddrive.

if (dir_exists("data_final")) dir_delete("data_final")
if (!dir_exists("data_final")) dir_create("data_final")

# write .Rds

tic()
write_rds(lyftbaywheels_final, "data_final/lyftbaywheels_final.Rds")
toc()

# write .csv

tic()
write_csv(lyftbaywheels_final, "data_final/lyftbaywheels_final.csv")
toc()

# write .feather from arrow R package

tic()
write_feather(lyftbaywheels_final, "data_final/lyftbaywheels_final.feather")
toc()

# write .parquet from arrow R package

tic()
write_parquet(lyftbaywheels_final, "data_final/lyftbaywheels_final.parquet")
toc()

# write .sqlite using the dbplyr R package

con <- DBI::dbConnect(RSQLite::SQLite(), dbname = "data_final/lyftbaywheels_final.sqlite")
copy_to(con, lyftbaywheels_final, "lyftbaywheels",
        temporary = FALSE,
        indexes = list(
          c("year", "month", "day"),
          "week_day",
          "start_station_id",
          "end_station_id"
        )
)

# write a disk.frame

# this will set up disk.frame with multiple workers
setup_disk.frame()
# this will allow unlimited amount of data to be passed from worker to worker
options(future.globals.maxSize = Inf)

# convert the flights data to a disk.frame and store the disk.frame in the folder
# "tmp_flights" and overwrite any content if needed
# Notice the use of the list.files() function.

lyftbaywheels_final_df <- as.disk.frame(
  lyftbaywheels_final, 
  outdir = file.path("data_final/data_disk_frame", "tmp_lyftbaywheels_final_df"),
  overwrite = TRUE)
lyftbaywheels_final_df

class(lyftbaywheels_final_df)

nrow(lyftbaywheels_final_df)
ncol(lyftbaywheels_final_df)

head(lyftbaywheels_final_df)

is_disk.frame('data_final/data_disk_frame/tmp_lyftbaywheels_final_df')

# remove all files

dir_delete("data")
dir_delete("data2")

# sound to indicate end of the program

beep();beep();beep(sound=4)